@@ -5,18 +5,23 @@
 
 use crate::{
     config::scripts::ScriptId,
-    list::{Styles, TestInstanceId},
+    list::{OwnedTestInstanceId, Styles, TestInstanceId},
     reporter::events::{AbortStatus, StressIndex},
+    run_mode::NextestRunMode,
     write_str::WriteStr,
 };
 use camino::{Utf8Path, Utf8PathBuf};
 use console::AnsiCodeIterator;
+use nextest_metadata::TestCaseName;
 use owo_colors::{OwoColorize, Style};
-use std::{fmt, io, path::PathBuf, process::ExitStatus, time::Duration};
+use std::{fmt, io, ops::ControlFlow, path::PathBuf, process::ExitStatus, time::Duration};
+use swrite::{SWrite, swrite};
 use unicode_width::UnicodeWidthChar;
 
 /// Utilities for pluralizing various words based on count or plurality.
 pub mod plural {
+    use crate::run_mode::NextestRunMode;
+
     /// Returns "were" if `plural` is true, otherwise "was".
     pub fn were_plural_if(plural: bool) -> &'static str {
         if plural { "were" } else { "was" }
@@ -31,14 +36,33 @@ pub mod plural {
         }
     }
 
-    /// Returns "test" if `count` is 1, otherwise "tests".
-    pub fn tests_str(count: usize) -> &'static str {
-        tests_plural_if(count != 1)
+    /// Returns:
+    ///
+    /// * If `mode` is `Test`: "test" if `count` is 1, otherwise "tests".
+    /// * If `mode` is `Benchmark`: "benchmark" if `count` is 1, otherwise "benchmarks".
+    pub fn tests_str(mode: NextestRunMode, count: usize) -> &'static str {
+        tests_plural_if(mode, count != 1)
     }
 
-    /// Returns "tests" if `plural` is true, otherwise "test".
-    pub fn tests_plural_if(plural: bool) -> &'static str {
-        if plural { "tests" } else { "test" }
+    /// Returns:
+    ///
+    /// * If `mode` is `Test`: "tests" if `plural` is true, otherwise "test".
+    /// * If `mode` is `Benchmark`: "benchmarks" if `plural` is true, otherwise "benchmark".
+    pub fn tests_plural_if(mode: NextestRunMode, plural: bool) -> &'static str {
+        match (mode, plural) {
+            (NextestRunMode::Test, true) => "tests",
+            (NextestRunMode::Test, false) => "test",
+            (NextestRunMode::Benchmark, true) => "benchmarks",
+            (NextestRunMode::Benchmark, false) => "benchmark",
+        }
+    }
+
+    /// Returns "tests" or "benchmarks" based on the run mode.
+    pub fn tests_plural(mode: NextestRunMode) -> &'static str {
+        match mode {
+            NextestRunMode::Test => "tests",
+            NextestRunMode::Benchmark => "benchmarks",
+        }
     }
 
     /// Returns "binary" if `count` is 1, otherwise "binaries".
@@ -97,9 +121,35 @@ pub mod plural {
             "iterations"
         }
     }
+
+    /// Returns "run" if `count` is 1, otherwise "runs".
+    pub fn runs_str(count: usize) -> &'static str {
+        if count == 1 { "run" } else { "runs" }
+    }
+
+    /// Returns "orphan" if `count` is 1, otherwise "orphans".
+    pub fn orphans_str(count: usize) -> &'static str {
+        if count == 1 { "orphan" } else { "orphans" }
+    }
+
+    /// Returns "error" if `count` is 1, otherwise "errors".
+    pub fn errors_str(count: usize) -> &'static str {
+        if count == 1 { "error" } else { "errors" }
+    }
+
+    /// Returns "exists" if `count` is 1, otherwise "exist".
+    pub fn exist_str(count: usize) -> &'static str {
+        if count == 1 { "exists" } else { "exist" }
+    }
+
+    /// Returns "remains" if `count` is 1, otherwise "remain".
+    pub fn remains_str(count: usize) -> &'static str {
+        if count == 1 { "remains" } else { "remain" }
+    }
 }
 
-pub(crate) struct DisplayTestInstance<'a> {
+/// A helper for displaying test instances with formatting.
+pub struct DisplayTestInstance<'a> {
     stress_index: Option<StressIndex>,
     display_counter_index: Option<DisplayCounterIndex>,
     instance: TestInstanceId<'a>,
@@ -108,7 +158,8 @@ pub(crate) struct DisplayTestInstance<'a> {
 }
 
 impl<'a> DisplayTestInstance<'a> {
-    pub(crate) fn new(
+    /// Creates a new display formatter for a test instance.
+    pub fn new(
         stress_index: Option<StressIndex>,
         display_counter_index: Option<DisplayCounterIndex>,
         instance: TestInstanceId<'a>,
@@ -374,16 +425,31 @@ impl fmt::Display for DisplayStressIndex {
     }
 }
 
-pub(super) enum DisplayCounterIndex {
-    Counter { current: usize, total: usize },
-    Padded { character: char, width: usize },
+/// Counter index display for test instances.
+pub enum DisplayCounterIndex {
+    /// A counter with current and total counts.
+    Counter {
+        /// Current count.
+        current: usize,
+        /// Total count.
+        total: usize,
+    },
+    /// A padded display.
+    Padded {
+        /// Character to use for padding.
+        character: char,
+        /// Width to pad to.
+        width: usize,
+    },
 }
 
 impl DisplayCounterIndex {
+    /// Creates a new counter display.
     pub fn new_counter(current: usize, total: usize) -> Self {
         Self::Counter { current, total }
     }
 
+    /// Creates a new padded display.
     pub fn new_padded(character: char, width: usize) -> Self {
         Self::Padded { character, width }
     }
@@ -427,20 +493,25 @@ pub(crate) fn u32_decimal_char_width(n: u32) -> usize {
     (n.checked_ilog10().unwrap_or(0) + 1).try_into().unwrap()
 }
 
+pub(crate) fn u64_decimal_char_width(n: u64) -> usize {
+    // checked_ilog10 returns 0 for 1-9, 1 for 10-99, 2 for 100-999, etc. (And
+    // None for 0 which we unwrap to the same as 1). Add 1 to it to get the
+    // actual number of digits.
+    (n.checked_ilog10().unwrap_or(0) + 1).try_into().unwrap()
+}
+
 /// Write out a test name.
 pub(crate) fn write_test_name(
-    name: &str,
+    name: &TestCaseName,
     style: &Styles,
     writer: &mut dyn WriteStr,
 ) -> io::Result<()> {
-    // Look for the part of the test after the last ::, if any.
-    let mut splits = name.rsplitn(2, "::");
-    let trailing = splits.next().expect("test should have at least 1 element");
-    if let Some(rest) = splits.next() {
+    let (module_path, trailing) = name.module_path_and_name();
+    if let Some(module_path) = module_path {
         write!(
             writer,
             "{}{}",
-            rest.style(style.module_path),
+            module_path.style(style.module_path),
             "::".style(style.module_path)
         )?;
     }
@@ -451,26 +522,24 @@ pub(crate) fn write_test_name(
 
 /// Wrapper for displaying a test name with styling.
 pub(crate) struct DisplayTestName<'a> {
-    name: &'a str,
+    name: &'a TestCaseName,
     styles: &'a Styles,
 }
 
 impl<'a> DisplayTestName<'a> {
-    pub(crate) fn new(name: &'a str, styles: &'a Styles) -> Self {
+    pub(crate) fn new(name: &'a TestCaseName, styles: &'a Styles) -> Self {
         Self { name, styles }
     }
 }
 
 impl fmt::Display for DisplayTestName<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        // Look for the part of the test after the last ::, if any.
-        let mut splits = self.name.rsplitn(2, "::");
-        let trailing = splits.next().expect("test should have at least 1 element");
-        if let Some(rest) = splits.next() {
+        let (module_path, trailing) = self.name.module_path_and_name();
+        if let Some(module_path) = module_path {
             write!(
                 f,
                 "{}{}",
-                rest.style(self.styles.module_path),
+                module_path.style(self.styles.module_path),
                 "::".style(self.styles.module_path)
             )?;
         }
@@ -580,6 +649,131 @@ impl fmt::Display for FormattedDuration {
     }
 }
 
+#[derive(Debug)]
+pub(crate) struct FormattedRelativeDuration(pub(crate) Duration);
+
+impl fmt::Display for FormattedRelativeDuration {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        // Adapted from
+        // https://github.com/atuinsh/atuin/blob/bd2a54e1b1/crates/atuin/src/command/client/search/duration.rs#L5,
+        // and used under the MIT license.
+        fn item(unit: &'static str, value: u64) -> ControlFlow<(&'static str, u64)> {
+            if value > 0 {
+                ControlFlow::Break((unit, value))
+            } else {
+                ControlFlow::Continue(())
+            }
+        }
+
+        // impl taken and modified from
+        // https://github.com/tailhook/humantime/blob/master/src/duration.rs#L295-L331
+        // Copyright (c) 2016 The humantime Developers
+        fn fmt(f: Duration) -> ControlFlow<(&'static str, u64), ()> {
+            let secs = f.as_secs();
+            let nanos = f.subsec_nanos();
+
+            let years = secs / 31_557_600; // 365.25d
+            let year_days = secs % 31_557_600;
+            let months = year_days / 2_630_016; // 30.44d
+            let month_days = year_days % 2_630_016;
+            let days = month_days / 86400;
+            let day_secs = month_days % 86400;
+            let hours = day_secs / 3600;
+            let minutes = day_secs % 3600 / 60;
+            let seconds = day_secs % 60;
+
+            let millis = nanos / 1_000_000;
+            let micros = nanos / 1_000;
+
+            // a difference between our impl and the original is that
+            // we only care about the most-significant segment of the duration.
+            // If the item call returns `Break`, then the `?` will early-return.
+            // This allows for a very concise impl
+            item("y", years)?;
+            item("mo", months)?;
+            item("d", days)?;
+            item("h", hours)?;
+            item("m", minutes)?;
+            item("s", seconds)?;
+            item("ms", u64::from(millis))?;
+            item("us", u64::from(micros))?;
+            item("ns", u64::from(nanos))?;
+            ControlFlow::Continue(())
+        }
+
+        match fmt(self.0) {
+            ControlFlow::Break((unit, value)) => write!(f, "{value}{unit}"),
+            ControlFlow::Continue(()) => write!(f, "0s"),
+        }
+    }
+}
+
+/// Characters used for terminal output theming.
+///
+/// Provides both ASCII and Unicode variants for horizontal bars, progress indicators,
+/// spinners, and tree display characters.
+#[derive(Clone, Debug)]
+pub struct ThemeCharacters {
+    hbar: char,
+    progress_chars: &'static str,
+    use_unicode: bool,
+}
+
+impl Default for ThemeCharacters {
+    fn default() -> Self {
+        Self {
+            hbar: '-',
+            progress_chars: "=> ",
+            use_unicode: false,
+        }
+    }
+}
+
+impl ThemeCharacters {
+    /// Switches to Unicode characters for richer terminal output.
+    pub fn use_unicode(&mut self) {
+        self.hbar = '─';
+        // https://mike42.me/blog/2018-06-make-better-cli-progress-bars-with-unicode-block-characters
+        self.progress_chars = "█▉▊▋▌▍▎▏ ";
+        self.use_unicode = true;
+    }
+
+    /// Returns the horizontal bar character.
+    pub fn hbar_char(&self) -> char {
+        self.hbar
+    }
+
+    /// Returns a horizontal bar of the specified width.
+    pub fn hbar(&self, width: usize) -> String {
+        std::iter::repeat_n(self.hbar, width).collect()
+    }
+
+    /// Returns the progress bar characters.
+    pub fn progress_chars(&self) -> &'static str {
+        self.progress_chars
+    }
+
+    /// Returns the tree branch character for non-last children: `├─` or `|-`.
+    pub fn tree_branch(&self) -> &'static str {
+        if self.use_unicode { "├─" } else { "|-" }
+    }
+
+    /// Returns the tree branch character for the last child: `└─` or `\-`.
+    pub fn tree_last(&self) -> &'static str {
+        if self.use_unicode { "└─" } else { "\\-" }
+    }
+
+    /// Returns the tree continuation line: `│ ` or `| `.
+    pub fn tree_continuation(&self) -> &'static str {
+        if self.use_unicode { "│ " } else { "| " }
+    }
+
+    /// Returns the tree space (no continuation): `  `.
+    pub fn tree_space(&self) -> &'static str {
+        "  "
+    }
+}
+
 // "exited with"/"terminated via"
 pub(crate) fn display_exited_with(exit_status: ExitStatus) -> String {
     match AbortStatus::extract(exit_status) {
@@ -650,18 +844,29 @@ pub(crate) fn display_nt_status(
     // set is going to be displayed anyway. This makes all possible displays
     // uniform.
     let bolded_status = format!("{:#010x}", nt_status.style(bold_style));
+
+    match windows_nt_status_message(nt_status) {
+        Some(message) => format!("{bolded_status}: {message}"),
+        None => bolded_status,
+    }
+}
+
+/// Returns the human-readable message for a Windows NT status code, if available.
+#[cfg(windows)]
+pub(crate) fn windows_nt_status_message(
+    nt_status: windows_sys::Win32::Foundation::NTSTATUS,
+) -> Option<smol_str::SmolStr> {
     // Convert the NTSTATUS to a Win32 error code.
     let win32_code = unsafe { windows_sys::Win32::Foundation::RtlNtStatusToDosError(nt_status) };
 
     if win32_code == windows_sys::Win32::Foundation::ERROR_MR_MID_NOT_FOUND {
         // The Win32 code was not found.
-        return bolded_status;
+        return None;
     }
 
-    format!(
-        "{bolded_status}: {}",
-        io::Error::from_raw_os_error(win32_code as i32)
-    )
+    Some(smol_str::SmolStr::new(
+        io::Error::from_raw_os_error(win32_code as i32).to_string(),
+    ))
 }
 
 #[derive(Copy, Clone, Debug)]
@@ -681,6 +886,41 @@ unsafe extern "C" {
     fn __nextest_external_symbol_that_does_not_exist();
 }
 
+/// Formats an interceptor (debugger or tracer) error message for too many tests.
+pub fn format_interceptor_too_many_tests(
+    cli_opt_name: &str,
+    mode: NextestRunMode,
+    test_count: usize,
+    test_instances: &[OwnedTestInstanceId],
+    list_styles: &Styles,
+    count_style: Style,
+) -> String {
+    let mut msg = format!(
+        "--{} requires exactly one {}, but {} {} were selected:",
+        cli_opt_name,
+        plural::tests_plural_if(mode, false),
+        test_count.style(count_style),
+        plural::tests_str(mode, test_count)
+    );
+
+    for test_instance in test_instances {
+        let display = DisplayTestInstance::new(None, None, test_instance.as_ref(), list_styles);
+        swrite!(msg, "\n  {}", display);
+    }
+
+    if test_count > test_instances.len() {
+        let remaining = test_count - test_instances.len();
+        swrite!(
+            msg,
+            "\n  ... and {} more {}",
+            remaining.style(count_style),
+            plural::tests_str(mode, remaining)
+        );
+    }
+
+    msg
+}
+
 #[inline]
 #[expect(dead_code)]
 pub(crate) fn statically_unreachable() -> ! {
@@ -706,4 +946,19 @@ mod test {
         assert_eq!(3, usize_decimal_char_width(100));
         assert_eq!(3, usize_decimal_char_width(999));
     }
+
+    #[test]
+    fn test_u64_decimal_char_width() {
+        assert_eq!(1, u64_decimal_char_width(0));
+        assert_eq!(1, u64_decimal_char_width(1));
+        assert_eq!(1, u64_decimal_char_width(9));
+        assert_eq!(2, u64_decimal_char_width(10));
+        assert_eq!(2, u64_decimal_char_width(99));
+        assert_eq!(3, u64_decimal_char_width(100));
+        assert_eq!(3, u64_decimal_char_width(999));
+        assert_eq!(6, u64_decimal_char_width(999_999));
+        assert_eq!(7, u64_decimal_char_width(1_000_000));
+        assert_eq!(8, u64_decimal_char_width(10_000_000));
+        assert_eq!(8, u64_decimal_char_width(11_000_000));
+    }
 }
