@@ -10,7 +10,9 @@ use crate::{
         scripts::{WrapperScriptConfig, WrapperScriptTargetRunner},
     },
     double_spawn::DoubleSpawnInfo,
-    errors::{CreateTestListError, FromMessagesError, WriteTestListError},
+    errors::{
+        CreateTestListError, FromMessagesError, TestListFromSummaryError, WriteTestListError,
+    },
     helpers::{convert_build_platform, dylib_path, dylib_path_envvar, write_test_name},
     indenter::indented,
     list::{BinaryList, OutputFormat, RustBuildMeta, Styles, TestListState},
@@ -38,12 +40,14 @@ use nextest_metadata::{
 };
 use owo_colors::OwoColorize;
 use quick_junit::ReportUuid;
-use serde::Serialize;
+use serde::{Deserialize, Serialize};
 use std::{
-    borrow::Cow,
+    borrow::{Borrow, Cow},
     collections::{BTreeMap, BTreeSet},
     ffi::{OsStr, OsString},
-    fmt, io,
+    fmt,
+    hash::{Hash, Hasher},
+    io,
     path::PathBuf,
     sync::{Arc, OnceLock},
 };
@@ -206,6 +210,10 @@ pub struct SkipCounts {
     /// The number of skipped tests.
     pub skipped_tests: usize,
 
+    /// The number of skipped tests due to this being a rerun and the test was
+    /// already passing.
+    pub skipped_tests_rerun: usize,
+
     /// The number of tests skipped because they are not benchmarks.
     ///
     /// This is used when running in benchmark mode.
@@ -392,6 +400,98 @@ impl<'g> TestList<'g> {
         })
     }
 
+    /// Reconstructs a TestList from archived summary data.
+    ///
+    /// This is used during replay to reconstruct a TestList without
+    /// executing test binaries. The reconstructed TestList provides the
+    /// data needed for display through the reporter infrastructure.
+    pub fn from_summary(
+        graph: &'g PackageGraph,
+        summary: &TestListSummary,
+        mode: NextestRunMode,
+    ) -> Result<Self, TestListFromSummaryError> {
+        // Build RustBuildMeta from summary.
+        let rust_build_meta = RustBuildMeta::from_summary(summary.rust_build_meta.clone())
+            .map_err(TestListFromSummaryError::RustBuildMeta)?;
+
+        // Get the workspace root from the graph.
+        let workspace_root = graph.workspace().root().to_path_buf();
+
+        // Construct an empty environment map - we don't need it for replay.
+        let env = EnvironmentMap::empty();
+
+        // For replay, we don't need the actual dylib path since we're not executing tests.
+        let updated_dylib_path = OsString::new();
+
+        // Build test suites from the summary.
+        let mut rust_suites = IdOrdMap::new();
+        let mut test_count = 0;
+
+        for (binary_id, suite_summary) in &summary.rust_suites {
+            // Look up the package in the graph by package_id.
+            let package_id = PackageId::new(suite_summary.binary.package_id.clone());
+            let package = graph.metadata(&package_id).map_err(|_| {
+                TestListFromSummaryError::PackageNotFound {
+                    name: suite_summary.package_name.clone(),
+                    package_id: suite_summary.binary.package_id.clone(),
+                }
+            })?;
+
+            // Determine the status based on the summary.
+            let status = if suite_summary.status == RustTestSuiteStatusSummary::SKIPPED {
+                RustTestSuiteStatus::Skipped {
+                    reason: BinaryMismatchReason::Expression,
+                }
+            } else if suite_summary.status == RustTestSuiteStatusSummary::SKIPPED_DEFAULT_FILTER {
+                RustTestSuiteStatus::Skipped {
+                    reason: BinaryMismatchReason::DefaultSet,
+                }
+            } else {
+                // Build test cases from the summary (only for listed suites).
+                let test_cases: IdOrdMap<RustTestCase> = suite_summary
+                    .test_cases
+                    .iter()
+                    .map(|(name, info)| RustTestCase {
+                        name: name.clone(),
+                        test_info: info.clone(),
+                    })
+                    .collect();
+
+                test_count += test_cases.len();
+
+                // LISTED or any other status should be treated as listed.
+                RustTestSuiteStatus::Listed {
+                    test_cases: DebugIgnore(test_cases),
+                }
+            };
+
+            let suite = RustTestSuite {
+                binary_id: binary_id.clone(),
+                binary_path: suite_summary.binary.binary_path.clone(),
+                package,
+                binary_name: suite_summary.binary.binary_name.clone(),
+                kind: suite_summary.binary.kind.clone(),
+                non_test_binaries: BTreeSet::new(), // Not stored in summary.
+                cwd: suite_summary.cwd.clone(),
+                build_platform: suite_summary.binary.build_platform,
+                status,
+            };
+
+            let _ = rust_suites.insert_unique(suite);
+        }
+
+        Ok(Self {
+            rust_suites,
+            mode,
+            workspace_root,
+            env,
+            rust_build_meta,
+            updated_dylib_path,
+            test_count,
+            skip_counts: OnceLock::new(),
+        })
+    }
+
     /// Returns the total number of tests across all binaries.
     pub fn test_count(&self) -> usize {
         self.test_count
@@ -410,11 +510,18 @@ impl<'g> TestList<'g> {
     /// Returns the total number of skipped tests.
     pub fn skip_counts(&self) -> &SkipCounts {
         self.skip_counts.get_or_init(|| {
+            let mut skipped_tests_rerun = 0;
             let mut skipped_tests_non_benchmark = 0;
             let mut skipped_tests_default_filter = 0;
             let skipped_tests = self
                 .iter_tests()
                 .filter(|instance| match instance.test_info.filter_match {
+                    FilterMatch::Mismatch {
+                        reason: MismatchReason::RerunAlreadyPassed,
+                    } => {
+                        skipped_tests_rerun += 1;
+                        true
+                    }
                     FilterMatch::Mismatch {
                         reason: MismatchReason::NotBenchmark,
                     } => {
@@ -450,6 +557,7 @@ impl<'g> TestList<'g> {
 
             SkipCounts {
                 skipped_tests,
+                skipped_tests_rerun,
                 skipped_tests_non_benchmark,
                 skipped_tests_default_filter,
                 skipped_binaries,
@@ -577,9 +685,10 @@ impl<'g> TestList<'g> {
     // Helper methods
     // ---
 
-    // Empty list for tests.
-    #[cfg(test)]
-    pub(crate) fn empty() -> Self {
+    /// Creates an empty test list.
+    ///
+    /// This is primarily for use in tests where a placeholder test list is needed.
+    pub fn empty() -> Self {
         Self {
             test_count: 0,
             mode: NextestRunMode::Test,
@@ -1417,12 +1526,15 @@ impl fmt::Display for TestInstanceId<'_> {
 }
 
 /// An owned version of [`TestInstanceId`].
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq, Hash, PartialOrd, Ord)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct OwnedTestInstanceId {
     /// The binary ID.
     pub binary_id: RustBinaryId,
 
     /// The name of the test.
+    #[serde(rename = "name")]
     pub test_name: TestCaseName,
 }
 
@@ -1446,6 +1558,63 @@ impl TestInstanceId<'_> {
     }
 }
 
+/// Trait to allow retrieving data from a set of [`OwnedTestInstanceId`] using a
+/// [`TestInstanceId`].
+///
+/// This is an implementation of the [borrow-complex-key-example
+/// pattern](https://github.com/sunshowers-code/borrow-complex-key-example).
+pub trait TestInstanceIdKey {
+    /// Converts self to a [`TestInstanceId`].
+    fn key<'k>(&'k self) -> TestInstanceId<'k>;
+}
+
+impl TestInstanceIdKey for OwnedTestInstanceId {
+    fn key<'k>(&'k self) -> TestInstanceId<'k> {
+        TestInstanceId {
+            binary_id: &self.binary_id,
+            test_name: &self.test_name,
+        }
+    }
+}
+
+impl<'a> TestInstanceIdKey for TestInstanceId<'a> {
+    fn key<'k>(&'k self) -> TestInstanceId<'k> {
+        *self
+    }
+}
+
+impl<'a> Borrow<dyn TestInstanceIdKey + 'a> for OwnedTestInstanceId {
+    fn borrow(&self) -> &(dyn TestInstanceIdKey + 'a) {
+        self
+    }
+}
+
+impl<'a> PartialEq for dyn TestInstanceIdKey + 'a {
+    fn eq(&self, other: &(dyn TestInstanceIdKey + 'a)) -> bool {
+        self.key() == other.key()
+    }
+}
+
+impl<'a> Eq for dyn TestInstanceIdKey + 'a {}
+
+impl<'a> PartialOrd for dyn TestInstanceIdKey + 'a {
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl<'a> Ord for dyn TestInstanceIdKey + 'a {
+    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
+        self.key().cmp(&other.key())
+    }
+}
+
+impl<'a> Hash for dyn TestInstanceIdKey + 'a {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.key().hash(state);
+    }
+}
+
 /// Context required for test execution.
 #[derive(Clone, Debug)]
 pub struct TestExecuteContext<'a> {
@@ -1476,8 +1645,9 @@ mod tests {
     use nextest_filtering::{CompiledExpr, Filterset, FiltersetKind, ParseContext};
     use nextest_metadata::{FilterMatch, MismatchReason, PlatformLibdirUnavailable, RustTestKind};
     use pretty_assertions::assert_eq;
-    use std::sync::LazyLock;
+    use std::{hash::DefaultHasher, sync::LazyLock};
     use target_spec::Platform;
+    use test_strategy::proptest;
 
     #[test]
     fn test_parse_test_list() {
@@ -2214,4 +2384,55 @@ mod tests {
         assert!(result.is_err());
         insta::assert_snapshot!("control_character_error", result.unwrap_err());
     }
+
+    // Proptest to verify that the `Borrow<dyn TestInstanceIdKey>` implementation for
+    // `OwnedTestInstanceId` is consistent with Eq, Ord, and Hash.
+    #[proptest]
+    fn test_instance_id_key_borrow_consistency(
+        owned1: OwnedTestInstanceId,
+        owned2: OwnedTestInstanceId,
+    ) {
+        // Create borrowed trait object references.
+        let borrowed1: &dyn TestInstanceIdKey = &owned1;
+        let borrowed2: &dyn TestInstanceIdKey = &owned2;
+
+        // Verify Eq consistency: owned equality must match borrowed equality.
+        assert_eq!(
+            owned1 == owned2,
+            borrowed1 == borrowed2,
+            "Eq must be consistent between OwnedTestInstanceId and dyn TestInstanceIdKey"
+        );
+
+        // Verify PartialOrd consistency.
+        assert_eq!(
+            owned1.partial_cmp(&owned2),
+            borrowed1.partial_cmp(borrowed2),
+            "PartialOrd must be consistent between OwnedTestInstanceId and dyn TestInstanceIdKey"
+        );
+
+        // Verify Ord consistency.
+        assert_eq!(
+            owned1.cmp(&owned2),
+            borrowed1.cmp(borrowed2),
+            "Ord must be consistent between OwnedTestInstanceId and dyn TestInstanceIdKey"
+        );
+
+        // Verify Hash consistency.
+        fn hash_value(x: &impl Hash) -> u64 {
+            let mut hasher = DefaultHasher::new();
+            x.hash(&mut hasher);
+            hasher.finish()
+        }
+
+        assert_eq!(
+            hash_value(&owned1),
+            hash_value(&borrowed1),
+            "Hash must be consistent for owned1 and its borrowed form"
+        );
+        assert_eq!(
+            hash_value(&owned2),
+            hash_value(&borrowed2),
+            "Hash must be consistent for owned2 and its borrowed form"
+        );
+    }
 }
