@@ -13,9 +13,9 @@ use crate::{
         scripts::ScriptId,
     },
     errors::{ChildError, ChildFdError, ChildStartError, ErrorList},
-    list::{TestInstanceId, TestList},
+    list::{OwnedTestInstanceId, TestInstanceId, TestList},
     runner::{StressCondition, StressCount},
-    test_output::{ChildExecutionOutput, ChildOutput},
+    test_output::{ChildExecutionOutput, ChildOutput, ChildSingleOutput},
 };
 use chrono::{DateTime, FixedOffset};
 use nextest_metadata::MismatchReason;
@@ -106,7 +106,7 @@ pub enum TestEventKind<'a> {
         program: String,
 
         /// The arguments to the program.
-        args: &'a [String],
+        args: Vec<String>,
 
         /// True if some output from the setup script is being passed through.
         no_capture: bool,
@@ -124,7 +124,7 @@ pub enum TestEventKind<'a> {
         program: String,
 
         /// The arguments to the program.
-        args: &'a [String],
+        args: Vec<String>,
 
         /// The amount of time elapsed since the start of execution.
         elapsed: Duration,
@@ -151,7 +151,7 @@ pub enum TestEventKind<'a> {
         program: String,
 
         /// The arguments to the program.
-        args: &'a [String],
+        args: Vec<String>,
 
         /// Whether the JUnit report should store success output for this script.
         junit_store_success_output: bool,
@@ -163,7 +163,7 @@ pub enum TestEventKind<'a> {
         no_capture: bool,
 
         /// The execution status of the setup script.
-        run_status: SetupScriptExecuteStatus,
+        run_status: SetupScriptExecuteStatus<ChildSingleOutput>,
     },
 
     // TODO: add events for BinaryStarted and BinaryFinished? May want a slightly different way to
@@ -216,7 +216,7 @@ pub enum TestEventKind<'a> {
         test_instance: TestInstanceId<'a>,
 
         /// The status of this attempt to run the test. Will never be success.
-        run_status: ExecuteStatus,
+        run_status: ExecuteStatus<ChildSingleOutput>,
 
         /// The delay before the next attempt to run the test.
         delay_before_next_attempt: Duration,
@@ -267,7 +267,7 @@ pub enum TestEventKind<'a> {
         junit_store_failure_output: bool,
 
         /// Information about all the runs for this test.
-        run_statuses: ExecutionStatuses,
+        run_statuses: ExecutionStatuses<ChildSingleOutput>,
 
         /// Current statistics for number of tests so far.
         current_stats: RunStats,
@@ -398,11 +398,34 @@ pub enum TestEventKind<'a> {
 
         /// Statistics for the run, or overall statistics for stress tests.
         run_stats: RunFinishedStats,
+
+        /// Tests that were expected to run but were not seen during this run.
+        ///
+        /// This is only set for reruns when some tests from the outstanding set
+        /// did not produce any events.
+        outstanding_not_seen: Option<TestsNotSeen>,
     },
 }
 
-/// Progress for a stress test.
+/// Tests that were expected to run but were not seen during a rerun.
 #[derive(Clone, Debug)]
+pub struct TestsNotSeen {
+    /// A sample of test instance IDs that were not seen, up to a reasonable
+    /// limit.
+    ///
+    /// This uses [`OwnedTestInstanceId`] rather than [`TestInstanceId`]
+    /// because the tests may not be present in the current test list (they
+    /// come from the expected outstanding set from a prior run).
+    pub not_seen: Vec<OwnedTestInstanceId>,
+
+    /// The total number of tests not seen (may exceed `not_seen.len()`).
+    pub total_not_seen: usize,
+}
+
+/// Progress for a stress test.
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "progress-type", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum StressProgress {
     /// This is a count-based stress run.
     Count {
@@ -435,10 +458,10 @@ impl StressProgress {
     pub fn remaining(&self) -> Option<StressRemaining> {
         match self {
             Self::Count {
-                total: StressCount::Count(total),
+                total: StressCount::Count { count },
                 elapsed: _,
                 completed,
-            } => total
+            } => count
                 .get()
                 .checked_sub(*completed)
                 .and_then(|remaining| NonZero::try_from(remaining).ok())
@@ -478,7 +501,9 @@ pub enum StressRemaining {
 }
 
 /// The index of the current stress run.
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct StressIndex {
     /// The 0-indexed index.
     pub current: u32,
@@ -488,7 +513,9 @@ pub struct StressIndex {
 }
 
 /// Statistics for a completed test run or stress run.
-#[derive(Clone, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "type", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum RunFinishedStats {
     /// A single test run was completed.
     Single(RunStats),
@@ -509,7 +536,9 @@ impl RunFinishedStats {
 }
 
 /// Statistics for a test run.
-#[derive(Copy, Clone, Default, Debug, Eq, PartialEq)]
+#[derive(Copy, Clone, Default, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct RunStats {
     /// The total number of tests that were expected to be run at the beginning.
     ///
@@ -606,7 +635,9 @@ impl RunStats {
                     kind: RunStatsFailureKind::SetupScript,
                 }
             } else {
-                FinalRunStats::Failed(RunStatsFailureKind::SetupScript)
+                FinalRunStats::Failed {
+                    kind: RunStatsFailureKind::SetupScript,
+                }
             }
         } else if self.setup_scripts_initial_count > self.setup_scripts_finished_count {
             FinalRunStats::Cancelled {
@@ -627,7 +658,7 @@ impl RunStats {
                     kind,
                 }
             } else {
-                FinalRunStats::Failed(kind)
+                FinalRunStats::Failed { kind }
             }
         } else if self.initial_run_count > self.finished_count {
             FinalRunStats::Cancelled {
@@ -644,7 +675,10 @@ impl RunStats {
         }
     }
 
-    pub(crate) fn on_setup_script_finished(&mut self, status: &SetupScriptExecuteStatus) {
+    pub(crate) fn on_setup_script_finished(
+        &mut self,
+        status: &SetupScriptExecuteStatus<ChildSingleOutput>,
+    ) {
         self.setup_scripts_finished_count += 1;
 
         match status.result {
@@ -670,7 +704,7 @@ impl RunStats {
         }
     }
 
-    pub(crate) fn on_test_finished(&mut self, run_statuses: &ExecutionStatuses) {
+    pub(crate) fn on_test_finished(&mut self, run_statuses: &ExecutionStatuses<ChildSingleOutput>) {
         self.finished_count += 1;
         // run_statuses is guaranteed to have at least one element.
         // * If the last element is success, treat it as success (and possibly flaky).
@@ -738,7 +772,9 @@ impl RunStats {
 }
 
 /// A type summarizing the possible outcomes of a test run.
-#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[serde(tag = "outcome", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum FinalRunStats {
     /// The test run was successful, or is successful so far.
     Success,
@@ -759,11 +795,16 @@ pub enum FinalRunStats {
     },
 
     /// At least one test failed.
-    Failed(RunStatsFailureKind),
+    Failed {
+        /// The kind of failure that occurred.
+        kind: RunStatsFailureKind,
+    },
 }
 
 /// Statistics for a stress run.
-#[derive(Clone, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct StressRunStats {
     /// The number of stress runs completed.
     pub completed: StressIndex,
@@ -809,7 +850,9 @@ pub enum StressFinalRunStats {
 }
 
 /// A type summarizing the step at which a test run failed.
-#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+#[serde(tag = "step", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum RunStatsFailureKind {
     /// The run was interrupted during setup script execution.
     SetupScript,
@@ -826,29 +869,58 @@ pub enum RunStatsFailureKind {
 }
 
 /// Information about executions of a test, including retries.
-#[derive(Clone, Debug)]
-pub struct ExecutionStatuses {
+///
+/// The type parameter `O` represents how test output is stored.
+#[derive(Clone, Debug, PartialEq, Eq, Serialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(O: proptest::arbitrary::Arbitrary + std::fmt::Debug + 'static))
+)]
+pub struct ExecutionStatuses<O> {
     /// This is guaranteed to be non-empty.
-    statuses: Vec<ExecuteStatus>,
+    #[cfg_attr(test, strategy(proptest::collection::vec(proptest::arbitrary::any::<ExecuteStatus<O>>(), 1..=3)))]
+    statuses: Vec<ExecuteStatus<O>>,
+}
+
+impl<'de, O: Deserialize<'de>> Deserialize<'de> for ExecutionStatuses<O> {
+    fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        // Deserialize as the wrapper struct that matches the Serialize output.
+        #[derive(Deserialize)]
+        #[serde(rename_all = "kebab-case")]
+        struct Helper<O> {
+            statuses: Vec<ExecuteStatus<O>>,
+        }
+
+        let helper = Helper::<O>::deserialize(deserializer)?;
+        if helper.statuses.is_empty() {
+            return Err(serde::de::Error::custom("expected non-empty statuses"));
+        }
+        Ok(Self {
+            statuses: helper.statuses,
+        })
+    }
 }
 
 #[expect(clippy::len_without_is_empty)] // RunStatuses is never empty
-impl ExecutionStatuses {
-    pub(crate) fn new(statuses: Vec<ExecuteStatus>) -> Self {
+impl<O> ExecutionStatuses<O> {
+    pub(crate) fn new(statuses: Vec<ExecuteStatus<O>>) -> Self {
+        debug_assert!(!statuses.is_empty(), "ExecutionStatuses must be non-empty");
         Self { statuses }
     }
 
     /// Returns the last execution status.
     ///
     /// This status is typically used as the final result.
-    pub fn last_status(&self) -> &ExecuteStatus {
+    pub fn last_status(&self) -> &ExecuteStatus<O> {
         self.statuses
             .last()
             .expect("execution statuses is non-empty")
     }
 
     /// Iterates over all the statuses.
-    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &'_ ExecuteStatus> + '_ {
+    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &'_ ExecuteStatus<O>> + '_ {
         self.statuses.iter()
     }
 
@@ -858,7 +930,7 @@ impl ExecutionStatuses {
     }
 
     /// Returns a description of self.
-    pub fn describe(&self) -> ExecutionDescription<'_> {
+    pub fn describe(&self) -> ExecutionDescription<'_, O> {
         let last_status = self.last_status();
         if last_status.result.is_success() {
             if self.statuses.len() > 1 {
@@ -886,42 +958,66 @@ impl ExecutionStatuses {
     }
 }
 
+impl<O> IntoIterator for ExecutionStatuses<O> {
+    type Item = ExecuteStatus<O>;
+    type IntoIter = std::vec::IntoIter<ExecuteStatus<O>>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.statuses.into_iter()
+    }
+}
+
+// TODO: Add Arbitrary impl for ExecutionStatuses when generic type support is added.
+// This requires ExecuteStatus<O> to implement Arbitrary with proper bounds.
+
 /// A description of test executions obtained from `ExecuteStatuses`.
 ///
 /// This can be used to quickly determine whether a test passed, failed or was flaky.
-#[derive(Copy, Clone, Debug)]
-pub enum ExecutionDescription<'a> {
+///
+/// The type parameter `O` represents how test output is stored.
+#[derive(Debug)]
+pub enum ExecutionDescription<'a, O> {
     /// The test was run once and was successful.
     Success {
         /// The status of the test.
-        single_status: &'a ExecuteStatus,
+        single_status: &'a ExecuteStatus<O>,
     },
 
     /// The test was run more than once. The final result was successful.
     Flaky {
         /// The last, successful status.
-        last_status: &'a ExecuteStatus,
+        last_status: &'a ExecuteStatus<O>,
 
         /// Previous statuses, none of which are successes.
-        prior_statuses: &'a [ExecuteStatus],
+        prior_statuses: &'a [ExecuteStatus<O>],
     },
 
     /// The test was run once, or possibly multiple times. All runs failed.
     Failure {
         /// The first, failing status.
-        first_status: &'a ExecuteStatus,
+        first_status: &'a ExecuteStatus<O>,
 
         /// The last, failing status. Same as the first status if no retries were performed.
-        last_status: &'a ExecuteStatus,
+        last_status: &'a ExecuteStatus<O>,
 
         /// Any retries that were performed. All of these runs failed.
         ///
         /// May be empty.
-        retries: &'a [ExecuteStatus],
+        retries: &'a [ExecuteStatus<O>],
     },
 }
 
-impl<'a> ExecutionDescription<'a> {
+// Manual Copy and Clone implementations to avoid requiring O: Copy/Clone, since
+// ExecutionDescription just stores references.
+impl<O> Clone for ExecutionDescription<'_, O> {
+    fn clone(&self) -> Self {
+        *self
+    }
+}
+
+impl<O> Copy for ExecutionDescription<'_, O> {}
+
+impl<'a, O> ExecutionDescription<'a, O> {
     /// Returns the status level for this `ExecutionDescription`.
     pub fn status_level(&self) -> StatusLevel {
         match self {
@@ -975,7 +1071,7 @@ impl<'a> ExecutionDescription<'a> {
     }
 
     /// Returns the last run status.
-    pub fn last_status(&self) -> &'a ExecuteStatus {
+    pub fn last_status(&self) -> &'a ExecuteStatus<O> {
         match self {
             ExecutionDescription::Success {
                 single_status: last_status,
@@ -991,7 +1087,9 @@ impl<'a> ExecutionDescription<'a> {
 /// This contains the formatted error messages, pre-computed from the execution
 /// output and result. Useful for record-replay scenarios where the rendering
 /// is done on the server.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct ErrorSummary {
     /// A short summary of the error, suitable for display in a single line.
     pub short_message: String,
@@ -1004,7 +1102,9 @@ pub struct ErrorSummary {
 ///
 /// This contains an error message heuristically extracted from test output,
 /// such as a panic message or error string.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct OutputErrorSlice {
     /// The extracted error slice as a string.
     pub slice: String,
@@ -1018,21 +1118,37 @@ pub struct OutputErrorSlice {
 /// This is the external-facing type used by reporters. The `result` field uses
 /// [`ExecutionResultDescription`], a platform-independent type that can be
 /// serialized and deserialized across platforms.
-#[derive(Clone, Debug)]
-pub struct ExecuteStatus {
+///
+/// The type parameter `O` represents how test output is stored:
+/// - [`ChildSingleOutput`]: Output stored in memory with lazy string conversion.
+/// - Other types may be used for serialization to archives.
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(O: proptest::arbitrary::Arbitrary + std::fmt::Debug + 'static))
+)]
+pub struct ExecuteStatus<O> {
     /// Retry-related data.
     pub retry_data: RetryData,
     /// The stdout and stderr output for this test.
-    pub output: ChildExecutionOutputDescription,
+    pub output: ChildExecutionOutputDescription<O>,
     /// The execution result for this test: pass, fail or execution error.
     pub result: ExecutionResultDescription,
     /// The time at which the test started.
+    #[cfg_attr(
+        test,
+        strategy(crate::reporter::test_helpers::arb_datetime_fixed_offset())
+    )]
     pub start_time: DateTime<FixedOffset>,
     /// The time it took for the test to run.
+    #[cfg_attr(test, strategy(crate::reporter::test_helpers::arb_duration()))]
     pub time_taken: Duration,
     /// Whether this test counts as slow.
     pub is_slow: bool,
     /// The delay will be non-zero if this is a retry and delay was specified.
+    #[cfg_attr(test, strategy(crate::reporter::test_helpers::arb_duration()))]
     pub delay_before_start: Duration,
     /// Pre-computed error summary, if available.
     ///
@@ -1051,18 +1167,31 @@ pub struct ExecuteStatus {
 /// This is the external-facing type used by reporters. The `result` field uses
 /// [`ExecutionResultDescription`], a platform-independent type that can be
 /// serialized and deserialized across platforms.
-#[derive(Clone, Debug)]
-pub struct SetupScriptExecuteStatus {
+///
+/// The type parameter `O` represents how test output is stored.
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(O: proptest::arbitrary::Arbitrary + std::fmt::Debug + 'static))
+)]
+pub struct SetupScriptExecuteStatus<O> {
     /// Output for this setup script.
-    pub output: ChildExecutionOutputDescription,
+    pub output: ChildExecutionOutputDescription<O>,
 
     /// The execution result for this setup script: pass, fail or execution error.
     pub result: ExecutionResultDescription,
 
     /// The time at which the script started.
+    #[cfg_attr(
+        test,
+        strategy(crate::reporter::test_helpers::arb_datetime_fixed_offset())
+    )]
     pub start_time: DateTime<FixedOffset>,
 
     /// The time it took for the script to run.
+    #[cfg_attr(test, strategy(crate::reporter::test_helpers::arb_duration()))]
     pub time_taken: Duration,
 
     /// Whether this script counts as slow.
@@ -1084,7 +1213,9 @@ pub struct SetupScriptExecuteStatus {
 /// A map of environment variables set by a setup script.
 ///
 /// Part of [`SetupScriptExecuteStatus`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct SetupScriptEnvMap {
     /// The map of environment variables set by the script.
     pub env_map: BTreeMap<String, String>,
@@ -1094,11 +1225,23 @@ pub struct SetupScriptEnvMap {
 // Child execution output description types
 // ---
 
-/// The result of executing a child process, in a serializable format.
+/// The result of executing a child process, generic over output storage.
 ///
-/// This is the external-facing counterpart to [`ChildExecutionOutput`].
-#[derive(Clone, Debug)]
-pub enum ChildExecutionOutputDescription {
+/// This is the external-facing counterpart to [`ChildExecutionOutput`]. The
+/// type parameter `O` represents how output is stored:
+///
+/// - [`ChildSingleOutput`]: Output stored in memory with lazy string caching.
+///   Used by reporter event types during live runs.
+/// - [`ZipStoreOutput`](crate::record::ZipStoreOutput): Reference to a file in
+///   a zip archive. Used for record/replay serialization.
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "type", rename_all = "kebab-case")]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(O: proptest::arbitrary::Arbitrary + std::fmt::Debug + 'static))
+)]
+pub enum ChildExecutionOutputDescription<O> {
     /// The process was run and the output was captured.
     Output {
         /// If the process has finished executing, the final state it is in.
@@ -1107,7 +1250,7 @@ pub enum ChildExecutionOutputDescription {
         result: Option<ExecutionResultDescription>,
 
         /// The captured output.
-        output: ChildOutput,
+        output: ChildOutputDescription<O>,
 
         /// Errors that occurred while waiting on the child process or parsing
         /// its output.
@@ -1118,7 +1261,7 @@ pub enum ChildExecutionOutputDescription {
     StartError(ChildStartErrorDescription),
 }
 
-impl ChildExecutionOutputDescription {
+impl<O> ChildExecutionOutputDescription<O> {
     /// Returns true if there are any errors in this output.
     pub fn has_errors(&self) -> bool {
         match self {
@@ -1136,10 +1279,55 @@ impl ChildExecutionOutputDescription {
     }
 }
 
+/// The output of a child process, generic over output storage.
+///
+/// This represents either split stdout/stderr or combined output. The `Option`
+/// wrappers distinguish between "not captured" (`None`) and "captured but
+/// empty" (`Some` with empty content).
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(
+    test,
+    derive(test_strategy::Arbitrary),
+    arbitrary(bound(O: proptest::arbitrary::Arbitrary + std::fmt::Debug + 'static))
+)]
+pub enum ChildOutputDescription<O> {
+    /// The output was split into stdout and stderr.
+    Split {
+        /// Standard output, or `None` if not captured.
+        stdout: Option<O>,
+        /// Standard error, or `None` if not captured.
+        stderr: Option<O>,
+    },
+
+    /// The output was combined into a single stream.
+    Combined {
+        /// The combined output.
+        output: O,
+    },
+}
+
+impl ChildOutputDescription<ChildSingleOutput> {
+    /// Returns the lengths of stdout and stderr in bytes.
+    ///
+    /// Returns `None` for each stream that wasn't captured.
+    pub fn stdout_stderr_len(&self) -> (Option<u64>, Option<u64>) {
+        match self {
+            Self::Split { stdout, stderr } => (
+                stdout.as_ref().map(|s| s.buf.len() as u64),
+                stderr.as_ref().map(|s| s.buf.len() as u64),
+            ),
+            Self::Combined { output } => (Some(output.buf.len() as u64), None),
+        }
+    }
+}
+
 /// A serializable description of an error that occurred while starting a child process.
 ///
 /// This is the external-facing counterpart to [`ChildStartError`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum ChildStartErrorDescription {
     /// An error occurred while creating a temporary path for a setup script.
     TempPath {
@@ -1176,7 +1364,9 @@ impl std::error::Error for ChildStartErrorDescription {
 /// A serializable description of an error that occurred while managing a child process.
 ///
 /// This is the external-facing counterpart to [`ChildError`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum ChildErrorDescription {
     /// An error occurred while reading standard output.
     ReadStdout {
@@ -1242,7 +1432,8 @@ impl std::error::Error for ChildErrorDescription {
 /// A serializable description of an I/O error.
 ///
 /// This captures the error message from an [`std::io::Error`].
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct IoErrorDescription {
     message: String,
 }
@@ -1255,7 +1446,7 @@ impl fmt::Display for IoErrorDescription {
 
 impl std::error::Error for IoErrorDescription {}
 
-impl From<ChildExecutionOutput> for ChildExecutionOutputDescription {
+impl From<ChildExecutionOutput> for ChildExecutionOutputDescription<ChildSingleOutput> {
     fn from(output: ChildExecutionOutput) -> Self {
         match output {
             ChildExecutionOutput::Output {
@@ -1264,7 +1455,7 @@ impl From<ChildExecutionOutput> for ChildExecutionOutputDescription {
                 errors,
             } => Self::Output {
                 result: result.map(ExecutionResultDescription::from),
-                output,
+                output: ChildOutputDescription::from(output),
                 errors: errors.map(|e| e.map(ChildErrorDescription::from)),
             },
             ChildExecutionOutput::StartError(error) => {
@@ -1274,6 +1465,18 @@ impl From<ChildExecutionOutput> for ChildExecutionOutputDescription {
     }
 }
 
+impl From<ChildOutput> for ChildOutputDescription<ChildSingleOutput> {
+    fn from(output: ChildOutput) -> Self {
+        match output {
+            ChildOutput::Split(split) => Self::Split {
+                stdout: split.stdout,
+                stderr: split.stderr,
+            },
+            ChildOutput::Combined { output } => Self::Combined { output },
+        }
+    }
+}
+
 impl From<ChildStartError> for ChildStartErrorDescription {
     fn from(error: ChildStartError) -> Self {
         match error {
@@ -1324,7 +1527,9 @@ impl From<ChildError> for ChildErrorDescription {
 }
 
 /// Data related to retries for a test.
-#[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub struct RetryData {
     /// The current attempt. In the range `[1, total_attempts]`.
     pub attempt: u32,
@@ -1492,6 +1697,7 @@ impl fmt::Debug for AbortStatus {
 /// platform-specific lookups.
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 #[non_exhaustive]
 pub enum AbortDescription {
     /// The process was aborted by a Unix signal.
@@ -1500,6 +1706,10 @@ pub enum AbortDescription {
         signal: i32,
         /// The signal name without the "SIG" prefix (e.g., "TERM", "SEGV"),
         /// if known.
+        #[cfg_attr(
+            test,
+            strategy(proptest::option::of(crate::reporter::test_helpers::arb_smol_str()))
+        )]
         name: Option<SmolStr>,
     },
 
@@ -1508,6 +1718,10 @@ pub enum AbortDescription {
         /// The NTSTATUS code.
         code: i32,
         /// The human-readable message from the Win32 error code, if available.
+        #[cfg_attr(
+            test,
+            strategy(proptest::option::of(crate::reporter::test_helpers::arb_smol_str()))
+        )]
         message: Option<SmolStr>,
     },
 
@@ -1569,6 +1783,7 @@ impl From<AbortStatus> for AbortDescription {
 /// This is the platform-independent counterpart to [`FailureStatus`].
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 #[non_exhaustive]
 pub enum FailureDescription {
     /// The test exited with a non-zero exit code.
@@ -1615,6 +1830,7 @@ impl fmt::Display for FailureDescription {
 /// in external-facing types like [`ExecuteStatus`].
 #[derive(Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
 #[serde(tag = "status", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 #[non_exhaustive]
 pub enum ExecutionResultDescription {
     /// The test passed.
@@ -1730,7 +1946,8 @@ impl From<ExecutionResult> for ExecutionResultDescription {
 
 // Note: the order here matters -- it indicates severity of cancellation
 /// The reason why a test run is being cancelled.
-#[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Deserialize, Serialize)]
+#[serde(rename_all = "kebab-case")]
 #[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum CancelReason {
     /// A setup script failed.
@@ -1808,7 +2025,7 @@ impl UnitKind {
 #[derive(Clone, Debug)]
 pub enum InfoResponse<'a> {
     /// A setup script's response.
-    SetupScript(SetupScriptInfoResponse<'a>),
+    SetupScript(SetupScriptInfoResponse),
 
     /// A test's response.
     Test(TestInfoResponse<'a>),
@@ -1816,7 +2033,7 @@ pub enum InfoResponse<'a> {
 
 /// A setup script's response to an information request.
 #[derive(Clone, Debug)]
-pub struct SetupScriptInfoResponse<'a> {
+pub struct SetupScriptInfoResponse {
     /// The stress index of the setup script.
     pub stress_index: Option<StressIndex>,
 
@@ -1827,13 +2044,13 @@ pub struct SetupScriptInfoResponse<'a> {
     pub program: String,
 
     /// The list of arguments to the program.
-    pub args: &'a [String],
+    pub args: Vec<String>,
 
     /// The state of the setup script.
     pub state: UnitState,
 
     /// Output obtained from the setup script.
-    pub output: ChildExecutionOutputDescription,
+    pub output: ChildExecutionOutputDescription<ChildSingleOutput>,
 }
 
 /// A test's response to an information request.
@@ -1852,7 +2069,7 @@ pub struct TestInfoResponse<'a> {
     pub state: UnitState,
 
     /// Output obtained from the test.
-    pub output: ChildExecutionOutputDescription,
+    pub output: ChildExecutionOutputDescription<ChildSingleOutput>,
 }
 
 /// The current state of a test or script process: running, exiting, or
@@ -2099,10 +2316,12 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::Test {
-                initial_run_count: 42,
-                not_run: 0
-            }),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::Test {
+                    initial_run_count: 42,
+                    not_run: 0,
+                },
+            },
             "failed => failure"
         );
         assert_eq!(
@@ -2113,10 +2332,12 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::Test {
-                initial_run_count: 42,
-                not_run: 0
-            }),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::Test {
+                    initial_run_count: 42,
+                    not_run: 0,
+                },
+            },
             "exec failed => failure"
         );
         assert_eq!(
@@ -2127,10 +2348,12 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::Test {
-                initial_run_count: 42,
-                not_run: 0
-            }),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::Test {
+                    initial_run_count: 42,
+                    not_run: 0,
+                },
+            },
             "timed out => failure {:?} {:?}",
             RunStats {
                 initial_run_count: 42,
@@ -2139,10 +2362,12 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::Test {
-                initial_run_count: 42,
-                not_run: 0
-            }),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::Test {
+                    initial_run_count: 42,
+                    not_run: 0,
+                },
+            },
         );
         assert_eq!(
             RunStats {
@@ -2178,7 +2403,9 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::SetupScript),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::SetupScript,
+            },
             "setup script failed => failure"
         );
         assert_eq!(
@@ -2189,7 +2416,9 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::SetupScript),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::SetupScript,
+            },
             "setup script exec failed => failure"
         );
         assert_eq!(
@@ -2200,7 +2429,9 @@ mod tests {
                 ..RunStats::default()
             }
             .summarize_final(),
-            FinalRunStats::Failed(RunStatsFailureKind::SetupScript),
+            FinalRunStats::Failed {
+                kind: RunStatsFailureKind::SetupScript,
+            },
             "setup script timed out => failure"
         );
         assert_eq!(
